<?php

/**
 * @file
 * Placeholder replacement tokens for commerce checkout url.
 */

use Drupal\commerce_checkout_url\LinkGenerator;
use Drupal\Core\Datetime\Entity\DateFormat;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Site\Settings;

/**
 * Implements hook_token_info().
 */
function commerce_checkout_url_token_info() {
  // Tokens for orders.
  $order = [];

  $order['commerce-checkout-url-expire'] = [
    'name' => t('Commerce Checkout URL expiry date'),
    'description' => t('The date when a one-time link to the checkout page generated now will expire.'),
  ];
  $order['commerce-checkout-url'] = [
    'name' => t('Commerce Checkout URL.'),
    'description' => t('A one-time link to the checkout page of an order protected with an authentication code.'),
  ];

  return [
    'tokens' => [
      'commerce-order' => $order,
    ],
  ];
}

/**
 * Implements hook_tokens().
 */
function commerce_checkout_url_tokens($type, $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata) {
  $token_service = \Drupal::token();

  $url_options = ['absolute' => TRUE];
  if (isset($options['langcode'])) {
    $url_options['language'] = \Drupal::languageManager()->getLanguage($options['langcode']);
    $langcode = $options['langcode'];
  }
  else {
    $langcode = LanguageInterface::LANGCODE_DEFAULT;
  }
  $replacements = [];

  if ($type === 'commerce-order' && !empty($data['commerce-order'])) {
    $order = $data['commerce-order'];
    $ttl = (int) Settings::get('commerce_checkout_url_ttl', LinkGenerator::URL_TTL_DEFAULT);
    $expire = \Drupal::time()->getRequestTime() + $ttl;

    foreach ($tokens as $name => $original) {
      switch ($name) {
        case 'commerce-checkout-url':
          /** @var \Drupal\commerce_checkout_url\LinkGeneratorInterface $linkGenerator */
          $linkGenerator = \Drupal::service('commerce_checkout_url.link_generator');
          $url = $linkGenerator->generate($order, $expire);
          $replacements[$original] = $url->setOptions($url_options)->toString();
          break;

        case 'commerce-checkout-url-expire':
          $date_format = DateFormat::load('medium');
          $bubbleable_metadata->addCacheableDependency($date_format);
          $replacements[$original] = \Drupal::service('date.formatter')->format($expire, 'medium', '', NULL, $langcode);
          break;
      }
    }

    if ($date_tokens = $token_service->findWithPrefix($tokens, 'commerce-checkout-url-expire')) {
      $replacements += $token_service->generate('date', $date_tokens, ['date' => $expire], $options, $bubbleable_metadata);
    }

  }

  return $replacements;
}
