<?php

namespace Drupal\commerce_checkout_url;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\Url;

class LinkGenerator implements LinkGeneratorInterface {

  /**
   * Default time to live for a checkout URL.
   */
  public const URL_TTL_DEFAULT = 604800;

  /**
   * Constructs a new link generator.
   *
   * @param \Drupal\commerce_checkout_url\SecurityKeyInterface $securityKey
   *   The commerce checkout url security key service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The request time service.
   */
  public function __construct(
    protected SecurityKeyInterface $securityKey,
    protected TimeInterface $time
  ) {
  }

  public function generate(OrderInterface $order, int $expire = NULL): Url {
    $result = FALSE;

    if (!isset($expire)) {
      $ttl = (int) Settings::get('commerce_checkout_url_ttl', self::URL_TTL_DEFAULT);
      $expire = $this->time->getRequestTime() + $ttl;
    }
    $expire_string = dechex($expire);

    $hmac = $this->securityKey->generate($order->id(), $expire_string);
    $result = Url::fromRoute('commerce_checkout_url.claim_form', [
      'commerce_order' => $order->id(),
      'expire' => $expire_string,
      'hmac' => $hmac,
    ]);

    return $result;
  }

}
